var Cus02500_MetaXCUstoRecalculado = function(General, GeneralServices, templateManager, ZhUtil, CUSUtilities, MessageService) {
  /*global Configuration, getLocalVar, moment, Constants*/
  var self = this;
  
  this.defaultValues = function(widget) {
    let firstReport = widget.getField('CD_REPORT_NAME').dataSource.getFirstRow();
    let currentMonthYear = moment().format('MM/YYYY');
    
    widget.currentRow.NMFILIALCENTRAL = getLocalVar('CDFILIAL') + " | " + getLocalVar('NMFILIAL');
    widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
    widget.currentRow.CD_REPORT_NAME = firstReport.value;
    widget.currentRow.NMFILIALAGREGADAS = 'T';
    widget.currentRow.NMGRUPSERV = 'T';
    widget.currentRow.NMSERVICO = 'T';
    
    widget.getField('DTINICIAL').setValue(currentMonthYear);
    widget.getField('DTFINAL').setValue(currentMonthYear);
    widget.getField('NMTIPOCUSTO').reload().then(callback => {
      let typeOfCost = callback.dataset.type_cost_lang;
      
      if(typeOfCost && typeOfCost.length){
        let currentCost = typeOfCost[0];
        widget.currentRow.NMTIPOCUSTO = currentCost.CDTIPOCUSTO + ' | ' + currentCost.NMTIPOCUSTO;
        widget.currentRow.CDTIPOCUSTO = currentCost.CDTIPOCUSTO;
      }
    });
  };
  
  this.onChangeUnidadeCentral = function(args) {
    let widget = args.owner.field.widget;
    
    widget.currentRow.CDFILIAL = widget.getField('NMFILIALCENTRAL').value() ? widget.getField('NMFILIALCENTRAL').value().substring(0,4) : '';
    
    widget.getField('NMSERVICO').clearValue();
    widget.getField('NMFILIALAGREGADAS').clearValue();
    
  };
  
  this.changeUniAgregFiltro = function(widget) {
    let fieldAgregadas = widget.getField('NMFILIALAGREGADAS');
    
    fieldAgregadas.dataSourceFilter = [];
    fieldAgregadas.dataSourceFilter = [
      {
        name: "NRORG",
        operator: "=" ,                   
        value: getLocalVar('NRORG')                    
      },
      {
        name: "P_CDOPERADOR",
        operator: "=" ,                   
        value: getLocalVar('CDOPERADOR')                    
      },
      {
        name: "P_CDFILCENTPME",
        operator: "=" ,                   
        value: widget.currentRow.CDFILIAL                
      }      
    ];
    
    fieldAgregadas.reload();
  };
  
  this.onChangeAgregadas = function(widget) {
    widget.getField('NMSERVICO').clearValue();
  };
  
  this.atualizaListaDeServico = function(widget) {
		let currentRow 		= widget.currentRow;
		let nmServicoField = widget.getField('NMSERVICO');
		let grupoServicos = '';
		let grupoUnidadesAgregadas = '';
		
		if(currentRow.CDFILIAL) {
		  grupoServicos = 'T';
		  grupoUnidadesAgregadas = 'T';
		}
  
		if (currentRow.NMGRUPSERV && Array.isArray(currentRow.NMGRUPSERV) && currentRow.NMGRUPSERV.length > 0) {
			grupoServicos = currentRow.NMGRUPSERV.join('_');
		} 
		
		if (currentRow.NMFILIALAGREGADAS && Array.isArray(currentRow.NMFILIALAGREGADAS) && currentRow.NMFILIALAGREGADAS.length > 0) {
			grupoUnidadesAgregadas = currentRow.NMFILIALAGREGADAS.join('_');
		}
		
		nmServicoField.dataSourceFilter = [
      {
        name: "P_CDFILCENTPME",
        operator: "=",
        value: currentRow.CDFILIAL
      },
      {
        name: "P_AGREGADAS",
        operator: "=",
        value: grupoUnidadesAgregadas || 'T'
      },      
      {
        name: "P_CDOPERADOR",
        operator: "=",
        value: getLocalVar('CDOPERADOR')
      },
      {
        name: "P_GRUPSERV",
        operator: "=",
        value: grupoServicos || 'T'
      },   
      {
        name: "NRORG",
        operator: "=",
        value: getLocalVar('NRORG')
      }
		];
		
		nmServicoField.reload();
  };
  
  this.getAllAgregadas = function(row) {
    return General.callRestEngine("cus#/crud/cus02500_unidades_agregadas", "FilterData", [
      {
        name: "NRORG",
        operator: "=" ,                   
        value: getLocalVar('NRORG')                    
      },
      {
        name: "P_CDOPERADOR",
        operator: "=" ,                   
        value: getLocalVar('CDOPERADOR')                    
      },
      {
        name: "P_CDFILCENTPME",
        operator: "=" ,                   
        value: row.CDFILIAL                
      }      
    ]);    
  };
  
  this.getAllServices = function(row) {
    var unidadesAgregadas = Array.isArray(row.NMFILIALAGREGADAS) ? row.NMFILIALAGREGADAS.join('_') : 'T';
    var nmGrupSer = (Array.isArray(row.NMGRUPSERV) && row.NMGRUPSERV.length > 0) ? row.NMGRUPSERV.join('_') : 'T';
    
    return General.callRestEngine("cus#/crud/servicos_por_unidades_por_grupserv", "FilterData", [
      {
        "name": "P_CDFILCENTPME",
        "operator": "=",
        "value": row.CDFILIAL
      },
      {
        "name": "P_AGREGADAS",
        "operator": "=",
        "value": unidadesAgregadas
      },      
      {
        "name": "P_CDOPERADOR",
        "operator": "=",
        "value": getLocalVar('CDOPERADOR')
      },
      {
        "name": "P_GRUPSERV",
        "operator": "=",
        "value": nmGrupSer
      },   
      {
        "name": "NRORG",
        "operator": "=",
        "value": getLocalVar('NRORG')
      }      
    ]);  
  };
  
  this.validaDatas = function (widget) {
    var dataInicial = moment(widget.currentRow.DTINICIAL, 'MM/YYYY');
    var dataFinal 	= moment(widget.currentRow.DTFINAL, 'MM/YYYY');    
    
    
    if (dataInicial && dataFinal && dataFinal < dataInicial) {
      MessageService.showMessageByCode('DATA_INICIAL_FINAL_INVALIDA', null, Constants.errorMessage);
      return false;
    }
    return true;
  };  
  
  this.validateFields = function(widget, row, currentRow) {
      
    if(!widget.getField('IDNAOESTOC').checked && !widget.getField('IDESTOCAVEL').checked) {
      MessageService.showMessageByCode('TIPO_PRODUTO_NAO_INFORMADO', null, Constants.errorMessage);
      return;
    }    
    
    var reportName = widget.currentRow.CD_REPORT_NAME;
    
    if(widget.isValid() && this.validaDatas(widget)) {
      var listaServFili = [];
      var listaServ = [];
      var listaAgregadas = [];
      row.LISTASERVFILI = [];
      
      if(row.NMFILIALAGREGADAS.length == 0 || row.NMFILIALAGREGADAS == "") {
        row.NMFILIALAGREGADAS = 'T';
      }
      
      if(row.NMSERVICO.length == 0 || row.NMSERVICO == "") {
        row.NMSERVICO = 'T';
      }      
      
      if(row.NMFILIALAGREGADAS == 'T') {
        self.getAllAgregadas(row).then(callback => {
          callback.dataset.cus02500_unidades_agregadas.forEach(unidade => {
            listaAgregadas.push(unidade.CDFILIAL);
          });
            row.LISTAFILIAISAGREGADAS = listaAgregadas;
        });        
      } else {
        row.LISTAFILIAISAGREGADAS = currentRow.NMFILIALAGREGADAS;
      }
      
      if(row.NMSERVICO == 'T') {
        self.getAllServices(row).then(callback => {
          callback.dataset.servicos_por_unidades_por_grupserv.forEach(servico => {
            listaServFili.push(servico.CDCHAVE);
            listaServ.push(servico.CDSERVFILI);
          });
          row.LISTASERVFILI = listaServFili;
        });
      } else {
        let listaChaveServFili = widget.currentRow.NMSERVICO;
        row.LISTASERVFILI = listaChaveServFili;
      }
      
		  row.dataInicial =  moment(row.DTINICIAL, 'MM/YYYY').startOf('month').format('DD/MM/YYYY');
		  row.dataFinal = moment(row.DTFINAL, 'MM/YYYY').endOf('month').format('DD/MM/YYYY');
      
  		var reportFormats = [
  	    {
          name: "pdf",
          value: "PDF"
  	    },
  	    {
          name: "xlsx",
          value: "XLSX"
  	    }
  		];
  	
  		General.openReportPopup(widget, this.abreRelatorio, [reportName, row, currentRow], reportFormats);  
    }
  };
  
  this.abreRelatorio = function(reportName, row, currentRow) {
    General.openReport('cus_relatorio#/Cus02500_MetaXCUstoRecalculado', reportName, row);
  }.bind(this);  

};

Configuration(function(ContextRegister) {
	ContextRegister.register('Cus02500_MetaXCUstoRecalculado', Cus02500_MetaXCUstoRecalculado);
});